(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (global = global || self, factory(global['modern-trello'] = {}));
}(this, function (exports) { 'use strict';

  const stringifyPrimitive = function (v) {
    switch (typeof v) {
      case 'string':
        return v;

      case 'boolean':
        return v ? 'true' : 'false';

      case 'number':
        return isFinite(v) ? v : '';

      default:
        return '';
    }
  };

  const stringify = (obj, sep, eq, name) => {
    sep = sep || '&';
    eq = eq || '=';

    if (obj === null) {
      obj = undefined;
    }

    if (typeof obj === 'object') {
      return Object.keys(obj).map(function (k) {
        var ks = encodeURIComponent(stringifyPrimitive(k)) + eq;

        if (Array.isArray(obj[k])) {
          return obj[k].map(function (v) {
            return ks + encodeURIComponent(stringifyPrimitive(v));
          }).join(sep);
        } else {
          return ks + encodeURIComponent(stringifyPrimitive(obj[k]));
        }
      }).filter(Boolean).join(sep);
    }

    if (!name) return '';
    return encodeURIComponent(stringifyPrimitive(name)) + eq + encodeURIComponent(stringifyPrimitive(obj));
  };

  class TrelloClient {
    constructor(options) {
      this.token = undefined;
      this.version = '1';
      this.authEndpoint = 'https://trello.com';

      this.authorizeURL = args => {
        const baseArgs = {
          response_type: 'token',
          key: this.key
        };
        const mergedOptions = Object.assign(baseArgs, args);
        return `${this.authEndpoint}/${this.version}/authorize?${stringify(mergedOptions)}`;
      };

      this.authorize = userOptions => {
        return new Promise(resolve => {
          const defaultOptions = {
            type: 'redirect',
            interactive: true,
            scope: ['read'],
            expiration: '30days'
          };
          const authorizeOptions = Object.assign(defaultOptions, userOptions);
          const regexToken = /[&#]?token=([0-9a-f]{64})/;

          if (this.token === undefined) {
            const match = regexToken.exec(window.location.hash);

            if (match) {
              this.token = match[1];
            }
          }

          if (this.token !== undefined) {
            location.hash = location.hash.replace(regexToken, '');
            return;
          }

          if (!authorizeOptions.interactive) {
            return;
          }

          const scope = (authorizeOptions.scope || []).join(',');

          switch (authorizeOptions.type) {
            case 'popup':
              (() => {
                const width = 550;
                const height = 725;
                const left = window.screenX + (window.innerWidth - width) / 2;
                const top = window.screenY + (window.innerHeight - height) / 2;
                const originMatch = new RegExp(`^[a-z]+://[^/]*`).exec(window.location.href);
                const origin = originMatch && originMatch[0];
                const authWindow = window.open(this.authorizeURL({
                  return_url: origin,
                  callback_method: 'postMessage',
                  scope,
                  expiration: authorizeOptions.expiration,
                  name: authorizeOptions.name
                }), 'trello', `width=${width},height=${height},left=${left},top=${top}`);

                const receiveMessage = event => {
                  if (event.origin !== this.authEndpoint || event.source !== authWindow) {
                    return;
                  }

                  if (event.source != null) {
                    event.source.close();
                  }

                  if (event.data != null && /[0-9a-f]{64}/.test(event.data)) {
                    this.token = event.data;
                  } else {
                    this.token = undefined;
                  }

                  if (typeof window.removeEventListener === 'function') {
                    window.removeEventListener('message', receiveMessage, false);
                  }

                  if (this.token !== undefined) {
                    resolve(this.token);
                  }
                }; // Listen for messages from the auth window


                if (typeof window.addEventListener === 'function') {
                  window.addEventListener('message', receiveMessage, false);
                }
              })();

              break;

            case 'redirect':
              window.location.href = this.authorizeURL({
                redirect_uri: location.href,
                callback_method: 'fragment',
                scope,
                expiration: authorizeOptions.expiration,
                name: authorizeOptions.name
              });
              break;

            default:
              window.location.href = this.authorizeURL({
                redirect_uri: location.href,
                callback_method: 'fragment',
                scope,
                expiration: authorizeOptions.expiration,
                name: authorizeOptions.name
              });
          }
        });
      };

      this.token = options.token;
      this.key = options.key;
    }

  }

  exports.default = TrelloClient;

}));
//# sourceMappingURL=modern-trello.umd.development.js.map
